-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local curRatio = 12
local minRatio = 2
local maxRatio = 12
local smoothRatio = 1
local highShiftUpRPM = 5000
local highShiftDownRPM = 4000
local lowShiftUpRPM = 2000
local lowShiftDownRPM = 1200
local n = 1

local function init()
	if v.data.engine.minRatio then
		minRatio = v.data.engine.minRatio
	end
	if v.data.engine.maxRatio then
		maxRatio = v.data.engine.maxRatio
	end
	if v.data.engine.maxRPM then
		maxRPM = v.data.engine.maxRPM
	end
	if v.data.engine.highShiftUpRPM then
		highShiftUpRPM = v.data.engine.highShiftUpRPM
	end
	if v.data.engine.highShiftDownRPM then
		highShiftDownRPM = v.data.engine.highShiftDownRPM
	end
	if v.data.engine.lowShiftUpRPM then
		lowShiftUpRPM = v.data.engine.lowShiftUpRPM
	end
	if v.data.engine.lowShiftDownRPM then
		lowShiftDownRPM = v.data.engine.lowShiftDownRPM
	end
	curRatio = maxRatio
end

local function updateGFX(dt)
	if v.data.engine.cvt == true then
		--continuously variable transmission (CVT)
		n = (((maxRPM + lowShiftDownRPM) - drivetrain.rpm)/325) - (drivetrain.avgAV * dt)
		
		curRatio = n * (input.throttle * input.throttle * input.throttle * input.throttle)
		
		if drivetrain.rpm < lowShiftDownRPM then
			curRatio = curRatio + (500 * dt)
		end
		
		--"low" gear
		if drivetrain.shifterPosition == 2 then 
			curRatio = maxRatio
		end
		
		smoothRatio = smoothRatio + (curRatio - smoothRatio) * dt * 1
		curRatio = smoothRatio
  
		--guard the values
		if curRatio > maxRatio then curRatio = maxRatio end
		if curRatio < minRatio then curRatio = minRatio end
		
		--disallow manual 1st gear
		if drivetrain.shifterPosition > 2 then 
			drivetrain.shifterPosition = 2
		end
		
		--send them to the gear ratio
		--print("curRatio"..curRatio)
		v.data.engine.gears = {curRatio, curRatio, [-1] = v.data.engine.gears[-1], [0] = 0}
	end
end

-- public interface
M.init      = init
M.updateGFX = updateGFX

return M